# Lovre Bogdanic, 19DEC2022
# Modified by Tetsuya Miwa, 09JAN2023

import ctcsound as csound
from matplotlib import pyplot as plt
from matplotlib import animation
import numpy as np

fftSize = 1024 # this needs to match gifftsiz in orc
fs = 48000 # sample rate

orc = '''
sr = 48000
0dbfs = 1
nchnls = 2

;general values for fourier transform
gifftsiz  =         1024   ; this needs to match above defined fftSize
gioverlap =         256
giwintyp  =         1 ;von hann window

;an 'empty' function table
giTableAmp  ftgen   1, 0, -(gifftsiz/2+1), 2, 0  ;number of analysis bin = gifftsiz/2 +1
giTableFreq ftgen   2, 0, -(gifftsiz/2+1), 2, 0  ;number of analysis bin = gifftsiz/2 +1

instr 1
    ;aout	oscili 0.5, 500, 1
    aout = oscili(0.5,line(100,p3,8000))
	aout	*= linseg:k(0, 1, 1, p3-1, 0)

    outs aout,aout

    kArr[] init gifftsiz+2 ;create array for bin data

    fsig	pvsanal aout, gifftsiz, gioverlap, gifftsiz, giwintyp
    kflag pvsftw fsig, giTableAmp, giTableFreq
endin
'''
sco = '''
f1 0 1024 7 1 512 1 0 -1 512 -1\n
i1 1 10\n
'''

cs = csound.Csound()
cs.setOption('-odac')
cs.compileOrc(orc)
cs.readScore(sco)
cs.start()

pt = csound.CsoundPerformanceThread(cs.csound())
pt.play()

fig, ax = plt.subplots()
ax.set(xlim=(0,fs/2), ylim=(0,1))
line, = ax.plot([], [], lw=2)

f_axis_delta = fs/fftSize # fundamental of analysis
f_axis = np.arange(0,fs/2 + f_axis_delta,f_axis_delta) # frequency bins
fftArrayAmp  = np.zeros(int(fftSize/2 + 1)) # array length must be fftSize/2 +1
fftArrayFreq = np.zeros(int(fftSize/2 + 1)) # array length must be fftSize/2 +1
def animate(i, f_axis_cs=[], amp_vals=[]):
    cs.tableCopyOut(1,fftArrayAmp)
    cs.tableCopyOut(2,fftArrayFreq)
    f_axis_cs = fftArrayFreq # estimated frequency by phase vocoder
    amp_vals = fftArrayAmp # fft amplitude values
    line.set_data(f_axis, amp_vals) # Use frequency bins for x axis value
    #line.set_data(f_axis_cs, amp_vals) # Use estimated frequency for x axis value

anim = animation.FuncAnimation(fig, animate, interval=100)
plt.show()